import json
import logging

from inspect_ai.solver import Generate, Solver, TaskState, solver

from astabench.evals.e2e_discovery.task import (
    get_hf_file_contents,
    print_and_log,
    record_token_counts,
)

logger = logging.getLogger(__name__)

FAKER_RESULTS_DIR = "e2e_discovery/faker/may-2025/reports/"


@solver
def faker_cached_solver() -> Solver:
    async def solve(state: TaskState, generate: Generate) -> TaskState:
        # TODO: Your real solver code here
        answer_json = await run_task(state.sample_id, state.input_text)
        state.output.completion = json.dumps(answer_json)
        return state

    return solve


async def run_task(sample_id, prompt, model="gpt4"):
    print_and_log(f"Retrieving cached Faker result for task {sample_id}...")
    results_file = FAKER_RESULTS_DIR + str(sample_id) + ".json"
    results_json = get_hf_file_contents(results_file)
    record_token_counts(results_json)
    return results_json
